#!/usr/bin/env python

import pytest
from markdownusm.parser import MarkdownParser


@pytest.mark.parametrize(
    "markdown, expected",
    [
        (
            "line1\nline2\nline3\n\n<!-- -->\nline4",
            4,
        ),
        ("""""", 0),
    ],
)
def test_constructor(markdown, expected):
    """Test line count"""
    obj = MarkdownParser(markdown=markdown)
    assert len(obj.lines) == expected


@pytest.mark.parametrize(
    "markdown, expected",
    [
        (
            "# Activity\n<!-- -->\n## Task1\n## Task2\nStory\n## Task3",
            ["Task1", "Task2", "Task3"],
        ),
        ("", []),
    ],
)
def test_extract_tasks(markdown, expected):
    actual = MarkdownParser(markdown=markdown)._extract_tasks()
    assert actual == expected


@pytest.mark.parametrize(
    "markdown, expected",
    [
        (
            "# Activity1\n## Task1\nStory\n\n## Task2\n# Activity2\n## Task3",
            {"Activity1": "Task1", "Activity2": "Task3"},
        )
    ],
)
def test_extract_activities_with_tasks(markdown, expected):
    actual = MarkdownParser(markdown=markdown)._extract_activities_with_tasks()
    assert actual == expected


@pytest.mark.parametrize(
    "markdown, expected",
    [
        (
            "- Release1\n- Release2\n<!-- -->\n\n- Release3\n-Release4",
            ["Release1", "Release2", "Release3"],
        ),
    ],
)
def test_extract_releases(markdown, expected):
    actual = MarkdownParser(markdown=markdown)._extract_releases()
    assert actual == expected


@pytest.mark.parametrize(
    "markdown, expected",
    [
        (
            "# Activity\n## Task1\nStory1\n# Activity\n## Task2\nStory2\n## Task3\n---\nStory3",
            [
                "## Task1",
                "Story1",
                "## Task2",
                "Story2",
                "## Task3",
                "---",
                "Story3",
            ],
        )
    ],
)
def test__extract_tasks_and_stories(markdown, expected):
    actual = MarkdownParser(markdown=markdown)._extract_tasks_and_stories()
    assert actual == expected


@pytest.mark.parametrize(
    "target, delimiter, expected",
    [
        (["Story1", "---", "Story2"], "---", [["Story1"], ["Story2"]]),
        (["Story1", "## Task1", "Story2"], "##", [["Story1"], ["Story2"]]),
        (["---", "Story", "Story"], "---", [[], ["Story", "Story"]]),
    ],
)
def test_divide_list_by_prefix(target, delimiter, expected):
    actual = MarkdownParser._divide_list_by_prefix(target, delimiter)
    assert actual == expected


@pytest.mark.parametrize(
    "markdown, expected",
    [
        (
            "## Task\nStory\nStory\nStory\n---\nStory\nStory\n---Story\nStory\n## Task\n---\n---\nStory\nStory\nStory",
            [3, 2, 3],
        )
    ],
)
def test_max_number_of_stories_in_each_release(markdown, expected):
    actual = MarkdownParser(markdown=markdown)._max_number_of_stories_in_each_release()
    assert actual == expected


@pytest.mark.parametrize(
    "markdown, expected",
    [
        (
            "# Activity1\n## Task1\nStory1\n## Task2\nStory2\n# Activity2\n## Task3",
            [
                {"text": "Activity1", "x": 0, "y": 0},
                {"text": "Activity2", "x": 2, "y": 0},
            ],
        )
    ],
)
def test_extract_activities_with_position(markdown, expected):
    actual = MarkdownParser(markdown=markdown).extract_activities_with_position()
    assert actual == expected


@pytest.mark.parametrize(
    "markdown, expected",
    [
        (
            "# Activity1\n## Task1\nStory1\n## Task2\nStory2\n# Activity2\n## Task3",
            [
                {"text": "Task1", "x": 0, "y": 1},
                {"text": "Task2", "x": 1, "y": 1},
                {"text": "Task3", "x": 2, "y": 1},
            ],
        )
    ],
)
def test_extract_tasks_with_position(markdown, expected):
    actual = MarkdownParser(markdown=markdown).extract_tasks_with_position()
    assert actual == expected


@pytest.mark.parametrize(
    "markdown, expected",
    [
        (
            "# Activity1\n## Task1\nStory1\n## Task2\nStory2\n# Activity2\n## Task3\n---\nStory3",
            [
                {"text": "Story1", "x": 0, "y": 2},
                {"text": "Story2", "x": 1, "y": 2},
                {"text": "Story3", "x": 2, "y": 3},
            ],
        )
    ],
)
def test_extract_stories_with_position(markdown, expected):
    actual = MarkdownParser(markdown=markdown).extract_stories_with_position()
    assert actual == expected


@pytest.mark.parametrize(
    "markdown, expected",
    [
        (
            "- Release1\n<!-- comment -->\n- Release2\n\n## Task\nStory\n## Task\n---\nStory\nStory\nStory",
            [
                {"text": "Release1", "x": -1, "y": 2},
                {"text": "Release2", "x": -1, "y": 3},
            ],
        )
    ],
)
def test_extract_release_texts_with_position(markdown, expected):
    actual = MarkdownParser(markdown=markdown).extract_release_texts_with_position()
    assert actual == expected


@pytest.mark.parametrize(
    "markdown, expected",
    [
        (
            "- Release1\n<!-- comment -->\n- Release2\n\n## Task\nStory\n## Task\n---\nStory\nStory\nStory\n## Task",
            [{"x": -1, "width": 3, "y": 1.9}, {"x": -1, "width": 3, "y": 2.9}],
        )
    ],
)
def test_create_release_bars_with_position(markdown, expected):
    actual = MarkdownParser(markdown=markdown).create_release_bars_with_position()
    assert actual == expected



#!/usr/bin/env python

import pytest
from markdownusm.usm import Usm
from markdownusm.xml import Rectangle


@pytest.mark.parametrize(
    "source, expected",
    [
        (
            [
                dict(text="Activity1", x=0, y=0),
                dict(text="Activity2", x=1, y=0),
            ],
            [
                Rectangle(
                    text="Activity1",
                    y=50.0,
                    x=290.0,
                    fillColor="#1F568A",
                    fontColor="#FFFFFF",
                    height=60,
                    width=140,
                    rounded=0,
                    white_space="wrap",
                    strokeColor="none",
                    horizontal_align="left",
                    vertical_align="top",
                    spacing_left=5,
                    spacing_right=5,
                    shadow=1,
                ),
                Rectangle(
                    text="Activity2",
                    y=50.0,
                    x=450.0,
                    fillColor="#1F568A",
                    fontColor="#FFFFFF",
                    height=60,
                    width=140,
                    rounded=0,
                    white_space="wrap",
                    strokeColor="none",
                    horizontal_align="left",
                    vertical_align="top",
                    spacing_left=5,
                    spacing_right=5,
                    shadow=1,
                ),
            ],
        )
    ],
)
def test_to_activity(source, expected):
    actual = Usm(source=source).to_activities()
    assert actual == expected


@pytest.mark.parametrize(
    "source, expected",
    [
        (
            [
                dict(text="Story 1", x=0, y=0),
            ],
            {
                "text": "Story 1",
                "x": "0",
                "y": "0",
                "fontColor": "#5F5F63",
                "fillColor": "#ebf4fa",
            },
        )
    ],
)
def test_set_story_properties(source, expected):
    usm = Usm(source=source)
    actual = usm._set_story_properties(usm.source[0])
    assert actual == expected


@pytest.mark.parametrize(
    "source, expected",
    [
        (
            [
                dict(text="Story 2!", x=1, y=0),
            ],
            {
                "text": "Story 2",
                "x": "1",
                "y": "0",
                "fontColor": "#5F5F63",
                "fillColor": "#f8d7da",
            },
        )
    ],
)
def test_set_story_properties_warning(source, expected):
    usm = Usm(source=source)
    actual = usm._set_story_properties(usm.source[0])
    assert actual == expected


@pytest.mark.parametrize(
    "source, expected",
    [
        (
            [
                dict(text="Story 1", x=0, y=0),
                dict(text="Story 2!", x=1, y=0),
            ],
            [
                Rectangle(
                    text="Story 1",
                    y=50.0,
                    x=290.0,
                    fillColor="#ebf4fa",
                    fontColor="#5F5F63",
                    height=60,
                    width=140,
                    rounded=0,
                    whiteSpace="wrap",
                    strokeColor="none",
                    align="left",
                    verticalAlign="top",
                    spacingLeft=5,
                    spacingRight=5,
                    shadow=1,
                ),
                Rectangle(
                    text="Story 2",
                    y=50.0,
                    x=450.0,
                    fillColor="#f8d7da",
                    fontColor="#5F5F63",
                    height=60,
                    width=140,
                    rounded=0,
                    whiteSpace="wrap",
                    strokeColor="none",
                    align="left",
                    verticalAlign="top",
                    spacingLeft=5,
                    spacingRight=5,
                    shadow=1,
                ),
            ],
        )
    ],
)
def test_to_stories(source, expected):
    actual = Usm(source=source).to_stories()
    assert actual == expected



#!/usr/bin/env python

from itertools import accumulate
from typing import Optional, Union

from pydantic import BaseModel, Field

U = Union[str, float]


class MarkdownParser(BaseModel):
    markdown: str = Field(title="Text written in markdown")

    # Lines without blank and comments, populated by constructor
    lines: Optional[list[str]] = Field(
        title="List of markdown lines without blanks and comments"
    )

    def __init__(self, **data):
        super().__init__(**data)

        # Remove blank lines and comments
        self.lines = list(
            filter(
                lambda x: x.strip() != "" and not x.startswith("<!--"),
                self.markdown.split("\n"),
            )
        )

    def extract_activities_with_position(self) -> list[dict[str, U]]:
        """Create list of dicts whose keys are activities' texts and positions
        
        Examples:
            >>> extract_activities_with_position()
            [
                {"text": "Activity1", "x": 0, "y": 0},
                {"text": "Activity2", "x": 2, "y": 0},
            ]

        """
        tasks: list[str] = self._extract_tasks()
        activities_with_tasks: dict[str, str] = self._extract_activities_with_tasks()

        return [
            dict(text=k, x=tasks.index(v), y=0)
            for k, v in activities_with_tasks.items()
        ]

    def extract_tasks_with_position(self) -> list[dict[str, U]]:
        """Create list of dicts whose keys are tasks' texts and positions

        Examples:
            >>> extract_tasks_with_position()
            [
                {"text": "Task1", "x": 0, "y": 1},
                {"text": "Task2", "x": 1, "y": 1},
                {"text": "Task3", "x": 2, "y": 1},
            ]

        """
        tasks: list[str] = self._extract_tasks()

        return [dict(text=task, x=i, y=1) for i, task in enumerate(tasks)]

    def extract_stories_with_position(self) -> list[dict[str, U]]:
        """Create list of dicts whose keys are stories' texts and positions

        Examples:
            >>> extract_stories_with_position()
            [
                {"text": "Story1", "x": 0, "y": 2},
                {"text": "Story2", "x": 1, "y": 2},
                {"text": "Story3", "x": 2, "y": 3},
            ]

        """

        # tasks_and_stories: ['## Task', 'Story', '---', 'Story', ...]
        tasks_and_stories = self._extract_tasks_and_stories()

        # [['Story1', 'Story2', '---', 'Story3'], ['---', 'Story4', 'Story5', '---', 'Story6'], ['Story7', '---', 'Story8']]
        split_by_task = self._divide_list_by_prefix(tasks_and_stories, "## ")[
            1:
        ]  # Remove first task

        """
        [
            [ # Task
                ['Story1', 'Story2'], # Release1
                ['Story3'] # Release2
            ],
            [
                [],
                ['Story4', 'Story5'],
                ['Story6']
            ],
            [
                ['Story7'],
                ['Story8']
            ]
        ]
        """

        # Get maximum number of stories to identify release bars positions
        number_of_stories_in_each_release = (
            self._max_number_of_stories_in_each_release()
        )
        # Adjustment in vertical position
        release_adjustment = list(
            accumulate([0] + number_of_stories_in_each_release[:-1])
        )

        split_by_release = list(
            map(lambda x: self._divide_list_by_prefix(x, "---"), split_by_task)
        )

        x = 0
        y = 2
        release_number = 0
        result: list[dict[str, U]] = []

        for task in split_by_release:
            release_number = 0
            for release in task:
                for story in release:
                    result.append(
                        dict(text=story, x=x, y=y + release_adjustment[release_number])
                    )
                    y += 1

                release_number += 1
                y = 2
            x += 1

        return result

    def extract_release_texts_with_position(
        self,
    ) -> list[dict[str, U]]:
        """
        [
            {"text": "Release1", "x": -1, "y": 1},
            {"text": "Release2", "x": -1, "y": 2},
        ]
        """
        releases = self._extract_releases()
        number_of_stories = self._max_number_of_stories_in_each_release()
        release_positions = list(accumulate([2] + number_of_stories[:-1]))

        return [
            dict(text=release, x=-1, y=y)
            for release, y in zip(releases, release_positions)
        ]

    def create_release_bars_with_position(self) -> list[dict[str, U]]:
        from itertools import accumulate

        number_of_stories = self._max_number_of_stories_in_each_release()
        number_of_stories_accumulate = list(accumulate([2] + number_of_stories[:-1]))

        number_of_tasks = len(self._extract_tasks())

        return [
            dict(x=-1, width=number_of_tasks, y=y - 0.1)
            for y in number_of_stories_accumulate
        ]

    def _extract_tasks(self) -> list[str]:
        """Create task list from markdown

        Examples:
            >>> _extract_tasks("#Activity\n## Task1\n## Task2")
            ['Task1', 'Task2', ...]

        """
        return list(
            map(
                lambda x: x.replace("## ", ""),
                filter(lambda x: x.startswith("## "), self.lines),
            )
        )

    def _extract_activities_with_tasks(self) -> dict[str, str]:
        """Create a dictionary whose keys are activities and values are tasks

        Examples:
            >>> _extract_activities_with_tasks("# Activity1\n## Task1\n## Task2")
            {'Activity1': 'Task1', 'Activity1': 'Task2', ...}

        """
        dic: dict[str, str] = {}
        for i, v in enumerate(self.lines):
            if not v.startswith("# "):
                continue

            dic = dic | {v.replace("# ", ""): self.lines[i + 1].replace("## ", "")}

        return dic

    def _extract_releases(self) -> list[str]:
        """Create release list from markdown

        Examples:
            >>> _extract_releases("- Release1\n- Release2\n# Activity\n## Task)
            ['Release1', 'Release2', ...]
        """
        return list(
            map(
                lambda x: x.replace("- ", ""),
                filter(lambda x: x.startswith("- "), self.lines),
            )
        )

    def _extract_tasks_and_stories(self) -> list[str]:
        """Create tasks(with #) and stories list from markdown

        Release separator will be included

        Examples:
            >>> _extract_tasks_and_stories("# Activity\n## Task1\nStory1\n---\nStory2")
            ['## Task1', 'Story1', '---', 'Story2', ...]

        """
        return list(
            filter(
                lambda x: not x.startswith("# ") and not x.startswith("- "), self.lines
            )
        )

    def _max_number_of_stories_in_each_release(self) -> list[float]:
        """Identify maximum number of stories in each release

        Examples:
            >>> _max_number_of_stories_in_each_release("## Task1\nStory1\n---Story2\n## Task2\n---\nStory3\nStory4")
            [1, 2]

        """
        # e.g. ['## Task1', 'Story1', '---', 'Story2', ...]
        tasks_and_stories = self._extract_tasks_and_stories()

        stories: list[list[str]] = []
        # Temporary list
        contents: list[str] = []
        for item in tasks_and_stories:
            if item.startswith("## "):
                if contents == []:
                    continue

                stories.append(contents)
                contents = []
            else:
                contents.append(item)

        # stories: [['Story1'], ['Story2'], ['---', 'Story3']]
        stories.append(contents)

        number_of_releases: int = (
            max([len([item for item in x if item.startswith("---")]) for x in stories])
            + 1
        )

        # number_of_tasks_in_each_release: [[1], [1], [0, 1]]
        number_of_tasks_in_each_release: list[list[int]] = [
            list(map(lambda x: len(x), self._divide_list_by_prefix(x, "---")))
            for x in stories
        ]

        # Arrange lists in length
        # number_of_tasks_in_releases: [[1, 0], [1, 0], [0, 1]]
        number_of_stories_in_releases = list(
            map(
                lambda x: self._extend_list(x, number_of_releases),
                number_of_tasks_in_each_release,
            )
        )

        # Extract maximum number of each release
        return list(map(lambda x: max(x), zip(*number_of_stories_in_releases)))

    @staticmethod
    def _divide_list_by_prefix(target: list[str], prefix: str) -> list[list[str]]:
        """Split a list into multiple lists wrapped as a list

        Examples:
            >>> _divide_list_by_prefix(["Story1", "---", "Story2"], "---")
            [["Story1"], ["Story2"]]

            >>> _divide_list_by_prefix(["---", "Story2"], "---")
            [[], ["Story2"]]

        """
        result: list[list[str]] = []
        child: list[str] = []

        for item in target:
            if item.startswith(prefix):
                result.append(child)
                child = []
            else:
                child.append(item)
        result.append(child)

        return result

    @staticmethod
    def _extend_list(
        source: list[int], disirable_length: int, complement: int = 0
    ) -> list[int]:
        """Extend list for arranging in numbers

        Example:
            >>> _extend_list([1, 2], 4)
            [1, 2, 0, 0]

        """
        if len(source) >= disirable_length:
            return source

        result = [0 for x in range(disirable_length)]

        for i in range(len(source)):
            result[i] = source[i]

        return result



#!/usr/bin/env python
"""Convert list objects to XML objects"""

from typing import Optional, Union

from pydantic import BaseModel

from markdownusm.xml import Line, Rectangle

U = Union[str, float]


class Usm(BaseModel):
    source: list[dict[str, U]]
    # Source list with absolute position
    source_abs: Optional[list[dict[str, U]]]

    start_x: float = 290
    start_y: float = 50

    width: float = 140
    height: float = 60

    padding: float = 20

    activity_fill_color: str = "#1F568A"
    activity_font_color: str = "#FFFFFF"

    task_fill_color: str = "#3288C4"
    task_font_color: str = "#FFFFFF"

    release_text_fill_color: str = "none"
    release_text_font_color: str = "#7B8EA0"
    release_text_stroke_color: str = "none"

    release_bar_stroke_color: str = "#5F5F63"

    story_font_color: str = "#5F5F63"
    story_default_fill_color: str = "#ebf4fa"
    story_warning_fill_color: str = "#f8d7da"

    def __init__(self, **data):
        super().__init__(**data)

        self.source_abs = [self._update_dic(dic) for dic in self.source]

    def to_activities(self) -> list[Rectangle]:
        colors = dict(
            fillColor=self.activity_fill_color, fontColor=self.activity_font_color
        )

        return [Rectangle(**(dic | colors)) for dic in self.source_abs]

    def to_tasks(self) -> list[Rectangle]:
        colors = dict(fillColor=self.task_fill_color, fontColor=self.task_font_color)

        return [Rectangle(**(dic | colors)) for dic in self.source_abs]

    def to_stories(self) -> list[Rectangle]:
        return [
            Rectangle(**(dic | self._set_story_properties(dic)))
            for dic in self.source_abs
        ]

    def to_release_texts(self) -> list[Rectangle]:
        colors = dict(
            fillColor=self.release_text_fill_color,
            fontColor=self.release_text_font_color,
            strokeColor=self.release_text_stroke_color,
        )

        return [Rectangle(**(dic | colors)) for dic in self.source_abs]

    def to_release_bars(self) -> list[Line]:
        colors = dict(strokeColor=self.release_bar_stroke_color, shadow=1)

        return [Line(**dic | colors) for dic in self.source_abs]

    def _relative_to_absolute_vertical(self, y: float) -> float:
        """Convert relative vertical position to absolute position"""
        return self.start_y + (self.height + self.padding) * y

    def _relative_to_absolute_horizontal(self, x: float) -> float:
        """Convert relative horizontal position to absolute position"""
        return self.start_x + (self.width + self.padding) * x

    def _relative_to_absolute_width(self, width: float) -> float:
        """Convert relative width to absolute width"""
        return self.start_x + (self.width + self.padding) * width

    def _update_dic(self, dic: dict[str, U]) -> dict[str, U]:
        """Update dictonary to convert positions

        Called from constructor

        """
        result = dic.copy()

        if (x := result.get("x")) is not None:
            result = result | dict(x=self._relative_to_absolute_horizontal(float(x)))

        if (y := result.get("y")) is not None:
            result = result | dict(y=self._relative_to_absolute_vertical(float(y)))

        if (width := result.get("width")) is not None:
            result = result | dict(width=self._relative_to_absolute_width(float(width)))

        return result

    def _set_story_properties(self, source_dic: dict[str, U]) -> dict[str, U]:
        """Set story properties

        Story suffix `!` express warning point

        Examples:
            >>> _set_story_properties(dict(text="Story X!", fillColor="#000000", ...))
            dict(text="Story X", fillColor="#f8d7da", ...)

            >>> _set_story_properties(dict(text="Story X", fillColor="#000000", ...))
            dict(text="Story X", fillColor="#ebf4fa", ...)

        """
        result: dict[str, U] = source_dic.copy()

        update_dic = dict(
            fontColor=self.story_font_color, fillColor=self.story_default_fill_color
        )
        story_text = str(result.get("text"))

        if story_text[-1] == "!":
            # If story's suffix is `!`
            update_dic = update_dic | dict(
                text=story_text[:-1], fillColor=self.story_warning_fill_color
            )
        elif "#" in story_text:
            # If story has hex color code in suffix
            update_dic = update_dic | dict(
                text=story_text.split("#")[0].strip(),
                fillColor="#" + story_text.split("#")[1].strip(),
            )

        return result | update_dic



#!/usr/bin/env python

import pytest
from markdownusm.parser import MarkdownParser
from markdownusm.usm import Usm
from markdownusm.xml import XMLObjects


@pytest.fixture
def markdown():
    return """
- Release1
- Release2
- Release3
- Release4
- Release5

<!-- Comment -->
# Activity
## Task
Story
Story
---
---
Story

## Task
---
Story
Story
Story
---
Story

# Activity
## Task
---
Story
---
Story

## Task
Story
Story
Story
Story
---
---
Story
Story
Story
"""


def test_integration(markdown):
    parser = MarkdownParser(markdown=markdown)

    # Create list of dictionaries
    activities_list = parser.extract_activities_with_position()
    tasks_list = parser.extract_tasks_with_position()
    stories_list = parser.extract_stories_with_position()
    release_texts_list = parser.extract_release_texts_with_position()
    release_bars_list = parser.create_release_bars_with_position()

    # Create XML objects
    usm_activities = Usm(source=activities_list)
    usm_tasks = Usm(source=tasks_list)
    usm_stories = Usm(source=stories_list)
    usm_release_texts = Usm(source=release_texts_list)
    usm_release_bars = Usm(source=release_bars_list)

    # Create XML documents
    activities = usm_activities.to_activities()
    tasks = usm_tasks.to_tasks()
    stories = usm_stories.to_stories()
    release_texts = usm_release_texts.to_release_texts()
    release_bars = usm_release_bars.to_release_bars()

    export = XMLObjects(
        shapes=activities + tasks + stories + release_texts + release_bars
    ).render()

    assert (
        export
        == """
    <mxfile>
        <diagram>
            <mxGraphModel dx="661" dy="316" grid="0" gridSize="10" guides="1" tooltips="1" connect="1" arrows="1" fold="1" page="1" pageScale="1" pageWidth="827" pageHeight="1169" math="0" shadow="0" background="#FFFFFF">
                <root>
                    <mxCell id="0"/>
                    <mxCell id="1" parent="0"/>
                    
                    
        <mxCell value="Activity" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#1F568A;strokeColor=none;fontColor=#FFFFFF;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="290.0" y="50.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Task" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#3288C4;strokeColor=none;fontColor=#FFFFFF;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="290.0" y="130.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Task" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#3288C4;strokeColor=none;fontColor=#FFFFFF;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="450.0" y="130.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Task" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#3288C4;strokeColor=none;fontColor=#FFFFFF;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="610.0" y="130.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Task" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#3288C4;strokeColor=none;fontColor=#FFFFFF;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="770.0" y="130.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Story" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#ebf4fa;strokeColor=none;fontColor=#5F5F63;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="290.0" y="210.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Story" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#ebf4fa;strokeColor=none;fontColor=#5F5F63;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="290.0" y="290.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Story" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#ebf4fa;strokeColor=none;fontColor=#5F5F63;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="290.0" y="770.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Story" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#ebf4fa;strokeColor=none;fontColor=#5F5F63;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="450.0" y="530.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Story" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#ebf4fa;strokeColor=none;fontColor=#5F5F63;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="450.0" y="610.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Story" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#ebf4fa;strokeColor=none;fontColor=#5F5F63;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="450.0" y="690.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Story" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#ebf4fa;strokeColor=none;fontColor=#5F5F63;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="450.0" y="770.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Story" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#ebf4fa;strokeColor=none;fontColor=#5F5F63;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="610.0" y="530.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Story" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#ebf4fa;strokeColor=none;fontColor=#5F5F63;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="610.0" y="770.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Story" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#ebf4fa;strokeColor=none;fontColor=#5F5F63;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="770.0" y="210.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Story" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#ebf4fa;strokeColor=none;fontColor=#5F5F63;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="770.0" y="290.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Story" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#ebf4fa;strokeColor=none;fontColor=#5F5F63;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="770.0" y="370.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Story" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#ebf4fa;strokeColor=none;fontColor=#5F5F63;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="770.0" y="450.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Story" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#ebf4fa;strokeColor=none;fontColor=#5F5F63;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="770.0" y="770.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Story" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#ebf4fa;strokeColor=none;fontColor=#5F5F63;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="770.0" y="850.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Story" style="html=1;rounded=0;whiteSpace=wrap;fillColor=#ebf4fa;strokeColor=none;fontColor=#5F5F63;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="770.0" y="930.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Release1" style="html=1;rounded=0;whiteSpace=wrap;fillColor=none;strokeColor=none;fontColor=#7B8EA0;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="130.0" y="210.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Release2" style="html=1;rounded=0;whiteSpace=wrap;fillColor=none;strokeColor=none;fontColor=#7B8EA0;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="130.0" y="530.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell value="Release3" style="html=1;rounded=0;whiteSpace=wrap;fillColor=none;strokeColor=none;fontColor=#7B8EA0;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1" parent="1" vertex="1">
            <mxGeometry x="130.0" y="770.0" width="140" height="60" as="geometry"/>
        </mxCell>
        
                    
                    
        <mxCell style="html=1;endArrow=none;shadow=0;strokeWidth=2;strokeColor=#5F5F63" edge="1" parent="1">
            <mxGeometry width="50" height="50" relative="1" as="geometry">
                <mxPoint x="130.0" y="202.0" as="sourcePoint"/>
                <mxPoint x="930.0" y="202.0" as="targetPoint"/>
            </mxGeometry>
        </mxCell>
        
                    
                    
        <mxCell style="html=1;endArrow=none;shadow=0;strokeWidth=2;strokeColor=#5F5F63" edge="1" parent="1">
            <mxGeometry width="50" height="50" relative="1" as="geometry">
                <mxPoint x="130.0" y="522.0" as="sourcePoint"/>
                <mxPoint x="930.0" y="522.0" as="targetPoint"/>
            </mxGeometry>
        </mxCell>
        
                    
                    
        <mxCell style="html=1;endArrow=none;shadow=0;strokeWidth=2;strokeColor=#5F5F63" edge="1" parent="1">
            <mxGeometry width="50" height="50" relative="1" as="geometry">
                <mxPoint x="130.0" y="762.0" as="sourcePoint"/>
                <mxPoint x="930.0" y="762.0" as="targetPoint"/>
            </mxGeometry>
        </mxCell>
        
                    
                </root>
            </mxGraphModel>
        </diagram>
    </mxfile>
    """
    )



#!/usr/bin/env python

from abc import ABCMeta, abstractmethod
from typing import Union

from jinja2 import BaseLoader, Environment
from pydantic import BaseModel, Field

U = Union[str, float]


class XMLObject(BaseModel, metaclass=ABCMeta):
    @abstractmethod
    def to_xml(self) -> str:
        pass


class Rectangle(XMLObject):
    """XML Rectangle Object"""

    text: str = Field(title="Text")

    y: float = Field(title="Vertical Position", ge=0)
    x: float = Field(title="Horizontal Position", ge=0)

    fillColor: str = Field(title="Fill Color")
    fontColor: str = Field(title="Font Color")

    height: float = Field(60, title="Height", gt=0)
    width: float = Field(140, title="Width", gt=0)

    rounded: int = Field(0, title="Rounded", ge=0, le=1)
    whiteSpace: str = Field("wrap", title="White Space")

    strokeColor: str = Field("none", title="Stroke Color")

    align: str = Field("left", title="Horizontal Align")
    verticalAlign: str = Field("top", title="Vertical Align")

    spacingLeft: float = Field(5, title="Left Spacing")
    spacingRight: float = Field(5, title="Right Spacing")
    shadow: int = Field(1, title="Shadow", ge=0, le=1)

    def to_xml(self) -> str:
        return f"""
        <mxCell value="{self.text}" style="{self._style()}" parent="1" vertex="1">
            <mxGeometry {self._geometry()} as="geometry"/>
        </mxCell>
        """

    def _geometry(self) -> str:
        """Position settings
        
        Examples:
            >>> _geometry()
            'x="0.0" y="0.0" width="140" height="60"'
        """
        dic = dict(x=self.x, y=self.y, width=self.width, height=self.height)
        return " ".join([f'{k}="{str(v)}"' for k, v in dic.items()])

    def _style(self) -> str:
        """Style settings"""
        dic = dict(
            html=1,
            rounded=self.rounded,
            whiteSpace=self.whiteSpace,
            fillColor=self.fillColor,
            strokeColor=self.strokeColor,
            fontColor=self.fontColor,
            align=self.align,
            verticalAlign=self.verticalAlign,
            spacingLeft=self.spacingLeft,
            spacingRight=self.spacingRight,
            shadow=self.shadow,
        )

        return ";".join([f"{k}={v}" for k, v in dic.items()])


class Line(XMLObject):
    """XML Line Object"""

    y: float = Field(title="Start/End vertical position")
    x: float = Field(title="Start horizontal positon")
    width: float = Field(title="Line length")

    strokeColor: str = Field("#000000")
    strokeWidth: float = Field(2)

    def _style(self) -> str:
        """Style settings"""
        dic = dict(
            html=1,
            endArrow="none",
            shadow=0,
            strokeWidth=self.strokeWidth,
            strokeColor=self.strokeColor,
        )

        return ";".join([f"{k}={v}" for k, v in dic.items()])

    def to_xml(self) -> str:
        return f"""
        <mxCell style="{self._style()}" edge="1" parent="1">
            <mxGeometry width="50" height="50" relative="1" as="geometry">
                <mxPoint x="{self.x}" y="{self.y}" as="sourcePoint"/>
                <mxPoint x="{self.width}" y="{self.y}" as="targetPoint"/>
            </mxGeometry>
        </mxCell>
        """


class XMLObjects(BaseModel):
    """Merge XML objects into one XML document"""

    shapes: list[XMLObject] = Field(title="XML Objects")

    xml_template = """
    <mxfile>
        <diagram>
            <mxGraphModel dx="661" dy="316" grid="0" gridSize="10" guides="1" tooltips="1" connect="1" arrows="1" fold="1" page="1" pageScale="1" pageWidth="827" pageHeight="1169" math="0" shadow="0" background="#FFFFFF">
                <root>
                    <mxCell id="0"/>
                    <mxCell id="1" parent="0"/>
                    {% for item in shapes %}
                    {{ item }}
                    {% endfor %}
                </root>
            </mxGraphModel>
        </diagram>
    </mxfile>
    """

    def render(self) -> str:
        """Render XML objects

        Returns:
            str: XML document

        """
        template = Environment(loader=BaseLoader()).from_string(self.xml_template)

        return template.render(shapes=self._shapes_to_xml())

    def _shapes_to_xml(self) -> list[str]:
        """Create XML string list"""
        return [x.to_xml() for x in self.shapes]



#!/usr/bin/env python

import argparse
import sys

from markdownusm.parser import MarkdownParser
from markdownusm.usm import Usm
from markdownusm.xml import XMLObjects

parser = argparse.ArgumentParser()
parser.add_argument(
    "file",
    type=argparse.FileType("r", encoding=sys.getdefaultencoding(), errors="ignore"),
)
parser.add_argument("-o", required=False, type=argparse.FileType("w", encoding="utf-8"))
args = parser.parse_args()


def main():
    xml = parse(args.file.read())

    if args.o is None:
        print(xml)
        return

    args.o.write(xml)


def parse(markdown: str) -> str:
    parser = MarkdownParser(markdown=markdown)

    activities_list = parser.extract_activities_with_position()
    tasks_list = parser.extract_tasks_with_position()
    stories_list = parser.extract_stories_with_position()
    release_texts_list = parser.extract_release_texts_with_position()
    release_bars_list = parser.create_release_bars_with_position()

    usm_activities = Usm(source=activities_list)
    usm_tasks = Usm(source=tasks_list)
    usm_stories = Usm(source=stories_list)
    usm_release_texts = Usm(source=release_texts_list)
    usm_release_bars = Usm(source=release_bars_list)

    activities = usm_activities.to_activities()
    tasks = usm_tasks.to_tasks()
    stories = usm_stories.to_stories()
    release_texts = usm_release_texts.to_release_texts()
    release_bars = usm_release_bars.to_release_bars()

    export = XMLObjects(
        shapes=activities + tasks + stories + release_texts + release_bars
    )

    return export.render()


if __name__ == "__main__":
    main()



#!/usr/bin/env python

from markdownusm.xml import Line, Rectangle


def test_rectangle():
    expected = "html=1;rounded=0;whiteSpace=wrap;fillColor=#000000;strokeColor=none;fontColor=#000000;align=left;verticalAlign=top;spacingLeft=5;spacingRight=5;shadow=1"
    actual = Rectangle(
        text="test", x=100, y=100, fillColor="#000000", fontColor="#000000"
    )._style()

    assert actual == expected


def test_lint():
    expected = "html=1;endArrow=none;shadow=0;strokeWidth=2;strokeColor=#000000"
    actual = Line(text="text", x=100, y=100, width=100)._style()

    assert actual == expected


def test__geometry():
    expected = 'x="0.0" y="0.0" width="140" height="60"'
    actual = Rectangle(text="", x=0, y=0, fillColor="", fontColor="")._geometry()

    assert actual == expected



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['markdownusm', 'markdownusm.tests']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=3.0.1,<4.0.0', 'pydantic>=1.8.2,<2.0.0']

entry_points = \
{'console_scripts': ['musm = markdownusm.cli:main']}

setup_kwargs = {
    'name': 'markdownusm',
    'version': '0.1.1.post2',
    'description': 'MarkdownUSM is the best way to draw a beautiful user story mapping diagram from simple markdown file.',
    'long_description': '# MarkdownUSM\n[![MIT License](http://img.shields.io/badge/license-MIT-blue.svg?style=flat)](LICENSE)\n[![CircleCI](https://circleci.com/gh/kbyky/markdownusm/tree/main.svg?style=shield&circle-token=33d038de0f7a2600f831702d67d7887b71f77eea)](https://circleci.com/gh/kbyky/markdownusm/tree/main)\n[![codecov](https://codecov.io/gh/kbyky/markdownusm/branch/main/graph/badge.svg?token=ZD51BWEICH)](https://codecov.io/gh/kbyky/markdownusm)\n[![Supported Versions](https://img.shields.io/pypi/pyversions/markdownusm.svg)](https://pypi.org/project/markdownusm)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Imports: isort](https://img.shields.io/badge/%20imports-isort-%231674b1?style=flat&labelColor=ef8336)](https://pycqa.github.io/isort/)\n\nMarkdownUSM is the best way to draw a beautiful user story mapping diagram from simple markdown file.\\\nMarkdown file will be converted to XML format then you can easily use the diagram on [draw.io](https://app.diagrams.net) and export in another format.\n\n![](https://github.com/kbyky/public/blob/main/img/markdownusm.svg?raw=true)\n\n## Installation\n```\n$ pip install markdownusm\n```\n\n## Examples\n\n### Create it\n\nCreate a file sample.md with:\n\n```\n<!-- Comment -->\n\n<!-- Release titles -->\n- Release 1\n- Release 2\n- Release 3\n- Release 4\n- Release 5\n\n# Activity 1\n## Task 1\nStory 1\n--- <!-- Release separator -->\nStory 2\n---\nStory 3\n\n## Task 2\n---\nStory 4\n\n<!-- Suffix `!` changes story postit color for warning -->\nStory 5!\n\n# Activity 2\n## Task 3\n---\n---\nStory 6 &lt;br&gt; Next line\n<!-- Story can change their colors by setting hex code following story title -->\nStory 7 #a6dfb5\n```\n\n### Run it\n\nThe simplest way with:\n\n```\n$ musm sample.md\n\n<mxfile>\n    <diagram>\n        <mxGraphModel dx="661" dy="316" grid="0" gridSize="10" guides="1" tooltips="1" connect="1" arrows="1" fold="1" page="1" pageScale="1" pageWidth="827" pageHeight="1169" math="0" shadow="0" background="#FFFFFF">\n        ...\n```\n\nOutput XML file with:\n```\n$ musm -o sample.dio sample.md\n```\n\n## License\nThis project is licensed under the terms of the MIT license.\n',
    'author': 'kbyky',
    'author_email': 'kbyky36@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/kbyky',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
